@@ -36,20 +36,16 @@
 
 #include <libempathy/empathy-idle.h>
 #include <libempathy/empathy-utils.h>
-#include <libempathy/empathy-debug.h>
 #include <libempathy/empathy-status-presets.h>
 
 #include "empathy-ui-utils.h"
 #include "empathy-images.h"
 #include "empathy-presence-chooser.h"
 
-#define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), EMPATHY_TYPE_PRESENCE_CHOOSER, EmpathyPresenceChooserPriv))
-
-#define DEBUG_DOMAIN "PresenceChooser"
-
 /* Flashing delay for icons (milliseconds). */
 #define FLASH_TIMEOUT 500
 
+#define GET_PRIV(obj) EMPATHY_GET_PRIV (obj, EmpathyPresenceChooser)
 typedef struct {
 	EmpathyIdle *idle;
 
@@ -100,8 +96,6 @@ static guint states[] = {MC_PRESENCE_AVAILABLE, TRUE,
 			 MC_PRESENCE_HIDDEN, FALSE,
 			 MC_PRESENCE_OFFLINE, FALSE};
 
-static void            empathy_presence_chooser_class_init      (EmpathyPresenceChooserClass *klass);
-static void            empathy_presence_chooser_init            (EmpathyPresenceChooser      *chooser);
 static void            presence_chooser_finalize               (GObject                    *object);
 static void            presence_chooser_presence_changed_cb    (EmpathyPresenceChooser      *chooser);
 static void            presence_chooser_reset_scroll_timeout   (EmpathyPresenceChooser      *chooser);
@@ -162,12 +156,12 @@ empathy_presence_chooser_class_init (EmpathyPresenceChooserClass *klass)
 static void
 empathy_presence_chooser_init (EmpathyPresenceChooser *chooser)
 {
-	EmpathyPresenceChooserPriv *priv;
-	GtkWidget                 *arrow;
-	GtkWidget                 *alignment;
-
-	priv = GET_PRIV (chooser);
+	GtkWidget                  *arrow;
+	GtkWidget                  *alignment;
+	EmpathyPresenceChooserPriv *priv = G_TYPE_INSTANCE_GET_PRIVATE (chooser,
+		EMPATHY_TYPE_PRESENCE_CHOOSER, EmpathyPresenceChooserPriv);
 
+	chooser->priv = priv;
 	gtk_button_set_relief (GTK_BUTTON (chooser), GTK_RELIEF_NONE);
 	gtk_button_set_focus_on_click (GTK_BUTTON (chooser), FALSE);
 
@@ -930,6 +924,7 @@ static void
 presence_chooser_dialog_show (void)
 {
 	GladeXML *glade;
+	gchar    *filename;
 
 	if (message_dialog) {
 		gtk_window_present (GTK_WINDOW (message_dialog->dialog));
@@ -937,7 +932,10 @@ presence_chooser_dialog_show (void)
 	}
 
 	message_dialog = g_new0 (CustomMessageDialog, 1);
-	glade = empathy_glade_get_file ("empathy-presence-chooser.glade",
+
+	filename = empathy_file_lookup ("empathy-presence-chooser.glade",
+					"libempathy-gtk");
+	glade = empathy_glade_get_file (filename,
 				       "custom_message_dialog",
 				       NULL,
 				       "custom_message_dialog", &message_dialog->dialog,
@@ -945,6 +943,8 @@ presence_chooser_dialog_show (void)
 				       "comboboxentry_message", &message_dialog->comboboxentry_message,
 				       "combobox_status", &message_dialog->combobox_status,
 				       NULL);
+	g_free (filename);
+
 	empathy_glade_connect (glade,
 			       message_dialog,
 			       "custom_message_dialog", "destroy", presence_chooser_dialog_destroy_cb,
