@@ -41,6 +41,7 @@
 #include <libempathy/empathy-contact.h>
 #include <libempathy/empathy-message.h>
 #include <libempathy/empathy-dispatcher.h>
+#include <libempathy/empathy-chatroom-manager.h>
 #include <libempathy/empathy-utils.h>
 
 #include <libempathy-gtk/empathy-images.h>
@@ -65,6 +66,7 @@ typedef struct {
 	gboolean     page_added;
 	gboolean     dnd_same_window;
 	guint        save_geometry_id;
+	EmpathyChatroomManager *chatroom_manager;
 	GtkWidget   *dialog;
 	GtkWidget   *notebook;
 
@@ -72,6 +74,7 @@ typedef struct {
 	GtkWidget   *menu_conv_clear;
 	GtkWidget   *menu_conv_insert_smiley;
 	GtkWidget   *menu_conv_contact;
+	GtkWidget   *menu_conv_favorite;
 	GtkWidget   *menu_conv_close;
 
 	GtkWidget   *menu_edit_cut;
@@ -223,6 +226,7 @@ chat_window_create_label (EmpathyChatWindow *window,
 
 	close_button = gtk_button_new ();
 	gtk_button_set_relief (GTK_BUTTON (close_button), GTK_RELIEF_NONE);
+	g_object_set_data (G_OBJECT (chat), "chat-window-tab-close-button", close_button);
 
 	/* We don't want focus/keynav for the button to avoid clutter, and
 	 * Ctrl-W works anyway.
@@ -285,8 +289,14 @@ chat_window_update (EmpathyChatWindow *window)
 	gboolean               is_connected;
 	gint                   num_pages;
 	gint                   page_num;
+	gint                   i;
 	const gchar           *name;
 	guint                  n_chats;
+	GdkPixbuf             *icon;
+	EmpathyContact        *remote_contact;
+	gboolean               avatar_in_icon;
+	GtkWidget             *chat;
+	GtkWidget             *chat_close_button;
 
 	/* Get information */
 	page_num = gtk_notebook_get_current_page (GTK_NOTEBOOK (priv->notebook));
@@ -323,7 +333,32 @@ chat_window_update (EmpathyChatWindow *window)
 		gtk_window_set_icon_name (GTK_WINDOW (priv->dialog),
 					  EMPATHY_IMAGE_MESSAGE);
 	} else {
-		gtk_window_set_icon_name (GTK_WINDOW (priv->dialog), NULL);
+		empathy_conf_get_bool (empathy_conf_get (),
+				       EMPATHY_PREFS_CHAT_AVATAR_IN_ICON,
+				       &avatar_in_icon);
+
+		if (n_chats == 1 && avatar_in_icon) {
+			remote_contact = empathy_chat_get_remote_contact (priv->current_chat);
+			icon = empathy_pixbuf_avatar_from_contact_scaled (remote_contact, 0, 0);
+			gtk_window_set_icon (GTK_WINDOW (priv->dialog), icon);
+			g_object_unref (icon);
+		} else {
+			gtk_window_set_icon_name (GTK_WINDOW (priv->dialog), NULL);
+		}
+	}
+
+	if (num_pages == 1) {
+		chat = gtk_notebook_get_nth_page (GTK_NOTEBOOK (priv->notebook), 0);
+		chat_close_button = g_object_get_data (G_OBJECT (chat),
+				"chat-window-tab-close-button");
+		gtk_widget_hide (chat_close_button);
+	} else {
+		for (i=0; i<num_pages; i++) {
+			chat = gtk_notebook_get_nth_page (GTK_NOTEBOOK (priv->notebook), i);
+			chat_close_button = g_object_get_data (G_OBJECT (chat),
+					"chat-window-tab-close-button");
+			gtk_widget_show (chat_close_button);
+		}
 	}
 }
 
@@ -460,6 +495,7 @@ chat_window_conv_activate_cb (GtkWidget         *menuitem,
 	EmpathyChatWindowPriv *priv = GET_PRIV (window);
 	GtkWidget             *submenu = NULL;
 
+	/* Contact submenu */
 	submenu = empathy_chat_get_contact_menu (priv->current_chat);
 	if (submenu) {
 		gtk_menu_item_set_submenu (GTK_MENU_ITEM (priv->menu_conv_contact),
@@ -469,6 +505,25 @@ chat_window_conv_activate_cb (GtkWidget         *menuitem,
 	} else {
 		gtk_widget_hide (priv->menu_conv_contact);
 	}
+
+	/* Favorite room menu */
+	if (empathy_chat_is_room (priv->current_chat)) {
+		const gchar *room;
+		McAccount   *account;
+		gboolean     found;
+
+		room = empathy_chat_get_id (priv->current_chat);
+		account = empathy_chat_get_account (priv->current_chat);
+		found = empathy_chatroom_manager_find (priv->chatroom_manager,
+						       account, room) != NULL;
+
+		DEBUG ("This room %s favorite", found ? "is" : "is not");
+		gtk_check_menu_item_set_active (
+			GTK_CHECK_MENU_ITEM (priv->menu_conv_favorite), found);
+		gtk_widget_show (priv->menu_conv_favorite);
+	} else {
+		gtk_widget_hide (priv->menu_conv_favorite);
+	}
 }
 
 static void
@@ -480,6 +535,38 @@ chat_window_clear_activate_cb (GtkWidget        *menuitem,
 	empathy_chat_clear (priv->current_chat);
 }
 
+static void
+chat_window_favorite_toggled_cb (GtkCheckMenuItem  *menuitem,
+				 EmpathyChatWindow *window)
+{
+	EmpathyChatWindowPriv *priv = GET_PRIV (window);
+	gboolean               active;
+	McAccount             *account;
+	const gchar           *room;
+	EmpathyChatroom       *chatroom;
+
+	active = gtk_check_menu_item_get_active (menuitem);
+	account = empathy_chat_get_account (priv->current_chat);
+	room = empathy_chat_get_id (priv->current_chat);
+
+	chatroom = empathy_chatroom_manager_find (priv->chatroom_manager,
+						  account, room);
+
+	if (active && !chatroom) {
+		const gchar *name;
+
+		name = empathy_chat_get_name (priv->current_chat);
+		chatroom = empathy_chatroom_new_full (account, room, name, FALSE);
+		empathy_chatroom_manager_add (priv->chatroom_manager, chatroom);
+		g_object_unref (chatroom);
+		return;
+	}
+	
+	if (!active && chatroom) {
+		empathy_chatroom_manager_remove (priv->chatroom_manager, chatroom);
+	}
+}
+
 static const gchar *
 chat_get_window_id_for_geometry (EmpathyChat *chat)
 {
@@ -1055,6 +1142,7 @@ chat_window_finalize (GObject *object)
 
 	DEBUG ("Finalized: %p", object);
 
+	g_object_unref (priv->chatroom_manager);
 	if (priv->save_geometry_id != 0) {
 		g_source_remove (priv->save_geometry_id);
 	}
@@ -1112,6 +1200,7 @@ empathy_chat_window_init (EmpathyChatWindow *window)
 				       "menu_conv_clear", &priv->menu_conv_clear,
 				       "menu_conv_insert_smiley", &priv->menu_conv_insert_smiley,
 				       "menu_conv_contact", &priv->menu_conv_contact,
+				       "menu_conv_favorite", &priv->menu_conv_favorite,
 				       "menu_conv_close", &priv->menu_conv_close,
 				       "menu_edit_cut", &priv->menu_edit_cut,
 				       "menu_edit_copy", &priv->menu_edit_copy,
@@ -1131,6 +1220,7 @@ empathy_chat_window_init (EmpathyChatWindow *window)
 			      "chat_window", "configure-event", chat_window_configure_event_cb,
 			      "menu_conv", "activate", chat_window_conv_activate_cb,
 			      "menu_conv_clear", "activate", chat_window_clear_activate_cb,
+			      "menu_conv_favorite", "toggled", chat_window_favorite_toggled_cb,
 			      "menu_conv_close", "activate", chat_window_close_activate_cb,
 			      "menu_edit", "activate", chat_window_edit_activate_cb,
 			      "menu_edit_cut", "activate", chat_window_cut_activate_cb,
@@ -1145,6 +1235,8 @@ empathy_chat_window_init (EmpathyChatWindow *window)
 
 	g_object_unref (glade);
 
+	priv->chatroom_manager = empathy_chatroom_manager_new ();
+
 	priv->notebook = gtk_notebook_new ();
  	gtk_notebook_set_group (GTK_NOTEBOOK (priv->notebook), "EmpathyChatWindow"); 
 	gtk_box_pack_start (GTK_BOX (chat_vbox), priv->notebook, TRUE, TRUE, 0);
@@ -1261,7 +1353,7 @@ empathy_chat_window_get_default (void)
 		chat_window = l->data;
 
 		dialog = empathy_chat_window_get_dialog (chat_window);
-		if (empathy_window_get_is_visible (GTK_WINDOW (GTK_WINDOW (dialog)))) {
+		if (empathy_window_get_is_visible (GTK_WINDOW (dialog))) {
 			/* Found a visible window on this desktop */
 			return chat_window;
 		}
@@ -1299,20 +1391,23 @@ empathy_chat_window_add_chat (EmpathyChatWindow *window,
 	/* Reference the chat object */
 	g_object_ref (chat);
 
-	empathy_geometry_load (chat_get_window_id_for_geometry (chat), &x, &y, &w, &h);
-
-	if (x >= 0 && y >= 0) {
-		/* Let the window manager position it if we don't have
-		 * good x, y coordinates.
-		 */
-		gtk_window_move (GTK_WINDOW (priv->dialog), x, y);
-	}
-
-	if (w > 0 && h > 0) {
-		/* Use the defaults from the glade file if we don't have
-		 * good w, h geometry.
-		 */
-		gtk_window_resize (GTK_WINDOW (priv->dialog), w, h);
+	/* If this window has just been created, position it */
+	if (priv->chats == NULL) {
+		empathy_geometry_load (chat_get_window_id_for_geometry (chat), &x, &y, &w, &h);
+		
+		if (x >= 0 && y >= 0) {
+			/* Let the window manager position it if we don't have
+			 * good x, y coordinates.
+			 */
+			gtk_window_move (GTK_WINDOW (priv->dialog), x, y);
+		}
+		
+		if (w > 0 && h > 0) {
+			/* Use the defaults from the glade file if we don't have
+			 * good w, h geometry.
+			 */
+			gtk_window_resize (GTK_WINDOW (priv->dialog), w, h);
+		}
 	}
 
 	child = GTK_WIDGET (chat);
